if [ -x /usr/bin/install-info ]; then
  install-info /usr/info/libext2fs.info.gz /usr/info/dir
fi
( cd usr/lib ; rm -rf libcom_err.so )
( cd usr/lib ; ln -sf /lib/libcom_err.so.2 libcom_err.so )
( cd usr/lib ; rm -rf libe2p.so )
( cd usr/lib ; ln -sf /lib/libe2p.so.2 libe2p.so )
( cd usr/lib ; rm -rf libext2fs.so )
( cd usr/lib ; ln -sf /lib/libext2fs.so.2 libext2fs.so )
( cd usr/lib ; rm -rf libss.so )
( cd usr/lib ; ln -sf /lib/libss.so.2 libss.so )
( cd usr/lib ; rm -rf libuuid.so )
( cd usr/lib ; ln -sf /lib/libuuid.so.1 libuuid.so )
#( cd sbin ; rm -rf fsck.ext3 )
#( cd sbin ; ln -sf e2fsck fsck.ext3 )
( cd sbin ; rm -rf mkfs.ext2 )
( cd sbin ; ln -sf mke2fs mkfs.ext2 )
( cd lib ; rm -rf libcom_err.so.2 )
( cd lib ; ln -sf libcom_err.so.2.0 libcom_err.so.2 )
( cd lib ; rm -rf libss.so.2 )
( cd lib ; ln -sf libss.so.2.0 libss.so.2 )
( cd lib ; rm -rf libe2p.so.2 )
( cd lib ; ln -sf libe2p.so.2.3 libe2p.so.2 )
( cd lib ; rm -rf libext2fs.so.2 )
( cd lib ; ln -sf libext2fs.so.2.4 libext2fs.so.2 )
( cd lib ; rm -rf libuuid.so.1 )
( cd lib ; ln -sf libuuid.so.1.2 libuuid.so.1 )
( cd sbin ; rm -rf e2label )
( cd sbin ; ln -sf tune2fs e2label )
